@@ -47,7 +47,7 @@ Unpack::Unpack(ComprDataIO *DataIO)
 
 Unpack::~Unpack()
 {
-  InitFilters30();
+  InitFilters30(false);
 
   if (Window!=NULL)
     free(Window);
@@ -91,7 +91,7 @@ void Unpack::Init(size_t WinSize,bool Solid)
   if (Grow && Fragmented)
     throw std::bad_alloc();
 
-  byte *NewWindow=(byte *)malloc(WinSize);
+  byte *NewWindow=Fragmented ? NULL : (byte *)malloc(WinSize);
 
   if (NewWindow==NULL)
     if (Grow || WinSize<0x1000000)
@@ -102,6 +102,11 @@ void Unpack::Init(size_t WinSize,bool Solid)
     }
     else
     {
+      if (Window!=NULL) // If allocated by preceding files.
+      {
+        free(Window);
+        Window=NULL;
+      }
       FragWindow.Init(WinSize);
       Fragmented=true;
     }
@@ -109,7 +114,7 @@ void Unpack::Init(size_t WinSize,bool Solid)
   if (!Fragmented)
   {
     // Clean the window to generate the same output when unpacking corrupt
-    // RAR files, which may access to unused areas of sliding dictionary.
+    // RAR files, which may access unused areas of sliding dictionary.
     memset(NewWindow,0,WinSize);
 
     // If Window is not NULL, it means that window size has grown.
@@ -117,7 +122,7 @@ void Unpack::Init(size_t WinSize,bool Solid)
     // RAR archiving code does not allow it in solid streams now,
     // but let's implement it anyway just in case we'll change it sometimes.
     if (Grow)
-      for (size_t I=1;I<MaxWinSize;I++)
+      for (size_t I=1;I<=MaxWinSize;I++)
         NewWindow[(UnpPtr-I)&(WinSize-1)]=Window[(UnpPtr-I)&(MaxWinSize-1)];
 
     if (Window!=NULL)
@@ -146,7 +151,7 @@ void Unpack::DoUnpack(int Method,bool Solid)
     case 29: // rar 3.x compression
       Unpack29(Solid);
       break;
-    case 0: // RAR 5.0 compression algorithm 0.
+    case 50: // RAR 5.0 compression algorithm.
 #ifdef RAR_SMP
       if (MaxUserThreads>1)
       {
@@ -179,9 +184,11 @@ void Unpack::UnpInitData(bool Solid)
     memset(&BlockTables,0,sizeof(BlockTables));
     UnpPtr=WrPtr=0;
     WriteBorder=Min(MaxWinSize,UNPACK_MAX_WRITE)&MaxWinMask;
-
-    InitFilters();
   }
+  // Filters never share several solid files, so we can safely reset them
+  // even in solid archive.
+  InitFilters();
+
   Inp.InitBitInput();
   WrittenFileSize=0;
   ReadTop=0;
